@@ -50,7 +50,7 @@
    <gw-type>
    class-name
    needs-result-var?
-   wrap-value-cg unwrap-value-cg destruct-value-cg
+   wrap-value-cg unwrap-value-cg destroy-value-cg
    pre-call-arg-cg pre-call-result-cg call-arg-cg post-call-result-cg
    post-call-arg-cg call-cg set-value-cg
    
@@ -127,9 +127,6 @@
 
 ;;;
 
-(define-class <gw-language> ()
-  (description #:getter description #:init-keyword #:description))
-
 ;; An <gw-item> is "something" that shows up in the generated
 ;; wrapper. The following generics are invoked on all items:
 ;;
@@ -154,14 +151,6 @@
 ;;; Types
 ;;;
 
-(define-class <gw-value> ()
-  (typespec #:getter typespec #:init-keyword #:typespec)
-  (var #:getter var #:init-keyword #:var)
-  (wrapped-var #:getter wrapped-var #:init-keyword #:wrapped-var))
-
-(define-class <gw-param> (<gw-value>)
-  (number #:getter number #:init-keyword #:number))
-
 (define-class <gw-type> (<gw-item>)
   (name #:getter name #:init-keyword #:name)
   (class-name #:accessor class-name
@@ -191,6 +180,11 @@
 ;;; Values
 ;;;
 
+(define-class <gw-value> ()
+  (typespec #:getter typespec #:init-keyword #:typespec)
+  (var #:getter var #:init-keyword #:var)
+  (wrapped-var #:getter wrapped-var #:init-keyword #:wrapped-var))
+
 (define-method (type (value <gw-value>))
   (type (typespec value)))
 
@@ -206,9 +200,9 @@
 
 (define-generic wrap-value-cg)
 (define-generic unwrap-value-cg)
-(define-generic destruct-value-cg)
+(define-generic destroy-value-cg)
 
-(define-method (destruct-value-cg (type <gw-type>) (value <gw-value>) err)
+(define-method (destroy-value-cg (type <gw-type>) (value <gw-value>) err)
   '())
 
 (define-method (pre-call-arg-cg (type <gw-type>) (param <gw-value>) err)
@@ -237,20 +231,24 @@
                                     status-var)
   (list
    (wrap-value-cg type result status-var)
-   (destruct-value-cg type result status-var)))
+   (destroy-value-cg type result status-var)))
 
 
-(define-method (post-call-arg-cg (type <gw-type>) (param <gw-param>) err)
+(define-method (post-call-arg-cg (type <gw-type>) (param <gw-value>) err)
   (list
    (if (memq 'out (options (typespec param)))
        (wrap-value-cg type param err)
        '())
-  (destruct-value-cg type param err)))
+  (destroy-value-cg type param err)))
 
 (define-method (set-value-cg (type <gw-type>) (lvalue <gw-value>) rvalue)
   (list (var lvalue) " = " rvalue ";\n"))
 
-;;;
+
+;;; Parameters
+
+(define-class <gw-param> (<gw-value>)
+  (number #:getter number #:init-keyword #:number))
 
 (define-method (visible? (self <gw-param>))
   (>= (number self) 0))
