@@ -481,7 +481,7 @@
                                 #f '(type arg-type range memory misc))
      "}\n")))
 
-(define-method (destruct-value-function-cg (type <gw-guile-rti-type>))
+(define-method (destroy-value-function-cg (type <gw-guile-rti-type>))
   
   (let* ((type-name (c-type-name type))
          (value (make <gw-rti-value>
@@ -489,10 +489,10 @@
                   #:typespec #f ;; the typespec is passed in the function
                   #:wrapped-var "value")))
     (list
-     "static void " (destruct-value-function-name type)
+     "static void " (destroy-value-function-name type)
      "(GWLangArena arena, void *instance, const GWTypeSpec *typespec, GWError *error) {\n"
      "  " (expand-special-forms
-           (destruct-value-cg type value "*error")
+           (destroy-value-cg type value "*error")
            #f '(type arg-type range memory misc))
      "}\n")))
 
@@ -670,7 +670,7 @@
      "else if (gw_wcp_is_of_type_p(" wct-var ", " sv "))\n"
      "  " c-var " = gw_wcp_get_ptr(" sv ");\n"
      "else\n"
-     `(gw:error ,status-var type ,sv))))
+     `(gw:error ,status-var type ,(wrapped-var value)))))
 
 (define-method (initializations-cg (wrapset <gw-wrapset>)
                                    (wct <gw-guile-wct>)
