@@ -50,6 +50,7 @@ char *alloca ();
 #define ARENA NULL /* Guile has no concept of an arena */
 
 static SCM the_scm_module = SCM_UNSPECIFIED;
+static SCM the_root_module = SCM_UNSPECIFIED;
 static SCM is_a_p_proc = SCM_UNSPECIFIED;
 static SCM module_add_x = SCM_UNSPECIFIED;
 static SCM scm_sym_make = SCM_UNSPECIFIED;
@@ -338,26 +339,22 @@ gw_scm_module_binder_proc (SCM module, SCM sym, SCM definep)
     proc_list = SCM_CDR (proc_list);
   }
 
+  /* To preserve the assertion that
+     (and (not (null? (hashq-ref latent-generics-hash generic-name #f)))
+          (not (defined? generics-name))),
+     remove the entry from the latent generics hash. */
+  scm_hashq_remove_x (latent_generics_hash, sym);
+
   var = scm_make_variable (generic);
   scm_call_3 (module_add_x, module, sym, var);
 
   return var;
 }
 
-void
-gw_guile_procedure_to_method_public (SCM proc, SCM class_name,
-                                     SCM generic_name,
-                                     SCM n_req_args, SCM use_optional_args)
-#define FUNC_NAME "%gw:procedure-to-method-public!"
+static void
+ensure_scm_module_hacked (void)
 {
   static int scm_module_hacked = 0;
-  SCM generic = SCM_BOOL_F;
-
-  SCM_VALIDATE_PROC (1, proc);
-  SCM_VALIDATE_SYMBOL (2, class_name);
-  SCM_VALIDATE_SYMBOL (3, generic_name);
-  SCM_VALIDATE_INUM (4, n_req_args);
-  /* the fifth is a bool */
   
   if (!scm_module_hacked)
   {
@@ -371,21 +368,45 @@ gw_guile_procedure_to_method_public (SCM proc, SCM class_name,
 
   if (SCM_FALSEP (latent_generics_hash))
     latent_generics_hash = scm_permanent_object (scm_c_make_hash_table (53));
+}
 
-  if (SCM_FALSEP (scm_hashq_ref (latent_generics_hash, generic_name,
-                                 SCM_BOOL_F)))
-    generic = scm_sym2var (generic_name,
-                           scm_module_lookup_closure (the_scm_module),
-                           SCM_BOOL_F);
+/* no explicit returns in this function */
+void
+gw_guile_procedure_to_method_public (SCM proc, SCM class_name,
+                                     SCM generic_name,
+                                     SCM n_req_args, SCM use_optional_args)
+#define FUNC_NAME "%gw:procedure-to-method-public!"
+{
+  SCM existing_binding = SCM_BOOL_F;
+  SCM existing_latents;
 
-  if (SCM_FALSEP (generic))
+  SCM_VALIDATE_PROC (1, proc);
+  SCM_VALIDATE_SYMBOL (2, class_name);
+  SCM_VALIDATE_SYMBOL (3, generic_name);
+  SCM_VALIDATE_INUM (4, n_req_args);
+  /* the fifth is a bool */
+  
+  ensure_scm_module_hacked ();
+
+  existing_latents = scm_hashq_ref (latent_generics_hash, generic_name,
+                                    SCM_EOL);
+  if (SCM_NULLP (existing_latents))
+    /* this means latent bindings for this variable have not been set up -- *
+       check now if there's an existing binding. use the root module to check --
+       prevents unnecessarily running our hacked scm module binder proc */
+    existing_binding =
+      scm_sym2var (generic_name,
+                   scm_module_lookup_closure (the_root_module),
+                   SCM_BOOL_F);
+
+  if (!SCM_NULLP (existing_latents) || SCM_FALSEP (existing_binding))
   {
-    /* Handle the common case when there's not already a symbol in the
-     * root. */
-    SCM entry;
-    SCM handle = scm_hashq_create_handle_x (latent_generics_hash, generic_name,
-                                            SCM_EOL);
-    entry = scm_c_make_vector (5, SCM_BOOL_F);
+    /* If there are already existing latent bindings, we just add ours onto the
+       list, knowing they will all be set up when the binding is forced.
+       
+       Otherwise, we're making the first latent binding, and there's nothing in
+       the root module that will conflict with our binding. */
+    SCM entry = scm_c_make_vector (5, SCM_BOOL_F);
     /* entry := #(proc class_name module n_req_args use_optional_args) */
     
     SCM_VECTOR_SET (entry, 0, proc);
@@ -393,30 +414,40 @@ gw_guile_procedure_to_method_public (SCM proc, SCM class_name,
     SCM_VECTOR_SET (entry, 2, scm_current_module ());
     SCM_VECTOR_SET (entry, 3, n_req_args);
     SCM_VECTOR_SET (entry, 4, use_optional_args);
-    SCM_SETCDR (handle, scm_cons (entry, SCM_CDR (handle)));
-    return;
+    
+    scm_hashq_set_x (latent_generics_hash, generic_name,
+                     scm_cons (entry, existing_latents));
   }
   else
   {
-    int is_generic = 0;
-    /* Otherwise, we have to make the generic. */
+    SCM val, generic = SCM_BOOL_F;
+    
+    /* !SCM_NULLP (existing_latents) implies SCM_FALSEP (existing_binding).
+       Thus we only get here if
+       SCM_NULLP (existing_latents) && SCM_NFALSEP (existing_binding). */
+    /* We have to make the generic. */
+
+    val = SCM_VARIABLE_REF (existing_binding);
 
-    generic = SCM_VARIABLE_REF (generic);
     /* I seem to remember this is_a_p thing is a hack around GOOPS's deficient
        macros, but I don't remember */
-    is_generic = SCM_NFALSEP (scm_call_2 (is_a_p_proc, generic,
-                                          scm_class_generic));
-
-    if (!is_generic)
+    if (SCM_NFALSEP (scm_call_2 (is_a_p_proc, val, scm_class_generic)))
     {
-      if (SCM_NFALSEP (scm_procedure_p (generic)))
+      /* The existing binding is a generic. Let's hang methods off of it. */
+      generic = val;
+    }
+    else
+    {
+      /* The existing binding is something else. We make a new generic, possibly
+         with a different name, and export it. */
+      /* NB: generics also satisfy procedure?. */
+      if (SCM_NFALSEP (scm_procedure_p (val)))
       {
         /* We need to fall back on the original binding. */
-        SCM default_val = generic;
         generic = scm_apply_0 (scm_sym_make,
                                scm_list_5 (scm_class_generic,
                                            k_name, generic_name,
-                                           k_default, default_val));
+                                           k_default, val));
       }
       else
       {
@@ -432,14 +463,13 @@ gw_guile_procedure_to_method_public (SCM proc, SCM class_name,
                               k_name, generic_name);
       }
       /* a rash and uncalled-for act */
-      scm_call_3 (module_add_x, the_scm_module, generic_name,
+      scm_call_3 (module_add_x, the_root_module, generic_name,
                   scm_make_variable (generic));
     }
+    gw_guile_add_subr_method (generic, proc, class_name, scm_current_module(),
+                              SCM_INUM (n_req_args),
+                              SCM_NFALSEP (use_optional_args));
   }
-
-  gw_guile_add_subr_method (generic, proc, class_name, scm_current_module(),
-                            SCM_INUM (n_req_args),
-                            SCM_NFALSEP (use_optional_args));
 }
 #undef FUNC_NAME
 
@@ -666,6 +696,10 @@ gw_guile_runtime_init (void)
                                                    "is-a?")));
     the_scm_module = scm_permanent_object (
             SCM_VARIABLE_REF (scm_c_lookup ("the-scm-module")));
+    the_root_module = scm_c_resolve_module ("guile");
+    /* the scm module is the interface for the root module, but there's no c
+       function to resolve interfaces. hack by referencing the variable
+       instead. */
     module_add_x = scm_permanent_object (
             SCM_VARIABLE_REF (scm_c_lookup ("module-add!")));
     k_specializers = scm_permanent_object (
